// Copyright (C) 2013 - 2016 - Michael Baudin
// Copyright (C) 2010 - DIGITEO - Michael Baudin
// Copyright (C) 1993 - 1995 - Anders Holtsberg
//
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt

function [f,xi,width]=ksdensity(varargin)
    // Kernel smoothing density estimate
    //  
    // Calling Sequence
    //   ksdensity(x)
    //   ksdensity(x,xi)
    //   ksdensity(x,xi,"kernel",akernel)
    //   ksdensity(x,xi,"npoints",npoints)
    //   ksdensity(x,xi,"support",support)
    //   ksdensity(x,xi,"width",width)
    //   ksdensity(x,xi,"func",func)
    //   f=ksdensity(...)
    //   [f,xi]=ksdensity(...)
    //   [f,xi,width]=ksdensity(...)
    //
    // Parameters
    // x : a n-by-1 matrix of doubles, the data
    // xi : a npoints-by-1 matrix of doubles, the points where the density is estimated
    // width : a 1-by-1 matrix of doubles, positive, the width of the kernel (default=Silverman's rule).
    // support : a 1-by-1 matrix of string, the support (default="unbounded"). If support is "unbounded", all real values are possible. If support is "positive", only positive values are possible. 
    // akernel : a 1-by-1 matrix of strings, the type of kernel (default="normal"). Available values are "normal", "biweight", "triangle", "epanechnikov".
    // npoints : a 1-by-1 matrix of doubles, positive, the number of points in the density estimate (default=100)
    // func : a string, the function to estimate (default="pdf"). Available values are "pdf" and "cdf". If func=="pdf", estimates the probability density function. If func=="cdf", estimates the cumulative distribution function. 
    // f : a npoints-by-1 matrix of doubles, the density estimate
    //
    // Description
    // Compute a kernel density estimate of the data. 
    // The density estimate f is evaluated at the points in xi.
    //
    // For i=1,...,npoints, the estimate for the PDF is :
    //
    // <latex>
    // f(i)= \frac{1}{nh} \sum_{j=1}^n K\left(\frac{xi(i)-x(j)}{h} \right)
    // </latex>
    //
    // where K is a kernel for PDFs. 
    // These kernels are available in the ksdensity_kernelpdf function.
    //
    // For i=1,...,npoints, the estimate for the CDF is :
    //
    // <latex>
    // f(i)= \frac{1}{n} \sum_{j=1}^n K\left(\frac{xi(i)-x(j)}{h} \right)
    // </latex>
    //
    // where K is a kernel for CDFs.
    // These kernels are available in the ksdensity_kernelcdf function.
    //
    // The Silverman rule for the width u of the kernel is:
    //
    // <latex>
    // u=1.06\hat{\sigma} n^{-1/5},
    // </latex>
    //
    // where <latex>\hat{\sigma}</latex> is the empirical standard 
    // deviation, and n is the size of the sample.
    //
    // On output, <literal>xi</literal> and <literal>f</literal> contain 
    // the kernel density estimate of the data. 
    // For i=1,2,...,npoints, f(i) is the estimate 
    // of the probability density at xi(i).
    //
    // More details on the kernels available in this function 
    // is provided in the help of ksdensity_kernel.
    //
    // Examples
    // X=distfun_normrnd(0,1,1000,1);
    // [f,xi,width]=ksdensity(X);
    // scf();
    // histo(X,[],%t);
    // plot(xi,f,"r-");
    // xtitle("Kernel density estimate","X","Density");
    // legend(["Data","PDF estimate"]);
    //
    // // Set the kernel width
    // X=distfun_normrnd(0,1,1000,1);
    // [f,xi,width]=ksdensity(X,"width",0.5);
    // scf();
    // histo(X,[],%t);
    // plot(xi,f,"r-");
    //
    // // Set the number of points
    // X=distfun_normrnd(0,1,1000,1);
    // [f,xi,width]=ksdensity(X,"npoints",500);
    // scf();
    // histo(X,[],%t);
    // plot(xi,f,"r-");
    //
    // // Set the kernel
    // scf();
    // X=distfun_normrnd(0,1,1000,1);
    // //
    // subplot(2,2,1);
    // histo(X,[],%t);
    // [f,xi,width]=ksdensity(X,"kernel","normal");
    // plot(xi,f,"r-");
    // xtitle("Gaussian Density Estimate","X","Density")
    // legend(["Data","PDF estimate"]);
    // //
    // subplot(2,2,2);
    // histo(X,[],%t);
    // [f,xi,width]=ksdensity(X,"kernel","epanechnikov");
    // plot(xi,f,"r-");
    // xtitle("Epanechnikov Density Estimate","X","Density")
    // legend(["Data","PDF estimate"]);
    // //
    // subplot(2,2,3);
    // histo(X,[],%t);
    // [f,xi,width]=ksdensity(X,"kernel","biweight");
    // plot(xi,f,"r-");
    // xtitle("Biweight Density Estimate","X","Density")
    // legend(["Data","PDF estimate"]);
    // //
    // subplot(2,2,4);
    // histo(X,[],%t);
    // [f,xi,width]=ksdensity(X,"kernel","triangle");
    // plot(xi,f,"r-");
    // xtitle("Triangular Density Estimate","X","Density")
    // legend(["Data","PDF estimate"]);
    //
    // // Set the kernel width
    // X=distfun_normrnd(0,1,1000,1);
    // scf();
    // //
    // [f,xi,width]=ksdensity(X,"width",0.1);
    // subplot(2,2,1)
    // histo(X,[],%t);
    // plot(xi,f,"r-");
    // xtitle("width=0.1")
    // //
    // [f,xi,width]=ksdensity(X,"width",0.25);
    // subplot(2,2,2)
    // histo(X,[],%t);
    // plot(xi,f,"r-");
    // xtitle("width=0.25")
    // //
    // [f,xi,width]=ksdensity(X,"width",0.5);
    // subplot(2,2,3)
    // histo(X,[],%t);
    // plot(xi,f,"r-");
    // xtitle("width=0.5")
    // //
    // [f,xi,width]=ksdensity(X,"width",1.);
    // subplot(2,2,4)
    // histo(X,[],%t);
    // plot(xi,f,"r-");
    // xtitle("width=1.")
    //
    // // Estimate CDF
    // X=distfun_normrnd(0,1,1000,1);
    // [f,xi,width]=ksdensity(X,"func","cdf","kernel","epanechnikov");
    // scf();
    // xlabel("x");
    // ylabel("P(X<x)");
    // X=gsort(X,"g","i");
    // n=size(X,"*");
    // p=(1:n)/n;
    // stairs(X,p);
    // plot(xi,f,"r-");
    // xtitle("Kernel CDF estimate","X","Density");
    // legend(["Data","CDF estimate"],2);
    //
    // Authors
    // Copyright (C) 2013 - 2016 - Michael Baudin
    // Copyright (C) 2010 - DIGITEO - Michael Baudin
    // Copyright (C) 1993 - 1995 - Anders Holtsberg

    function [f, xi, width] = %kdensity(x, xi, akernel, npoints, support, width, func)
        arguments
            x {mustBeA(x, "double"), mustBeVector}
            xi {mustBeA(xi, "double")} = []
            akernel (1,1) {mustBeA(akernel, "string"), mustBeMember(akernel, ["normal","biweight","triangle","epanechnikov"])} = "normal"
            npoints (1,1) {mustBeA(npoints, "double"), mustBeInteger, mustBeGreaterThanOrEqual(npoints, 1)} = 100
            support (1,1) {mustBeA(support, "string"), mustBeMember(support, ["unbounded","positive"])} = "unbounded"
            width {mustBeA(width, "double"), mustBeScalarOrEmpty} = []
            func (1,1) {mustBeA(func, "string"), mustBeMember(func, ["pdf","cdf"])} = "pdf"
        end

        if xi <> [] then
            error(msprintf(_("%s: Wrong size for input argument #%d: A vector expected.\n"), "ksendity", 2));
        end

        if width == [] then
            n = size(x, "*"); 
            width = 1.0600000000000001*stdev(x)*(n^(-1/5));
        end

        tiny=number_properties("tiny")

        if width < tiny then
            error(msprintf(_("%s: Wrong value for input argument ""%s"": A vector expected.\n"), "ksendity", "width"));
        end

        x=x(:)
        //
        if (support=="positive"&or(x<0)) then
            error(msprintf(gettext("%s: There is a negative element in X."),"ksdensity"));
        end
        //
        // Get the points where to estimate the density : xi
        if (xi==[]) then
            xi=linspace(min(x),max(x),npoints)'
        end    
        // Compute the density estimate value at point xi, 
        // given data in x, 
        // with kernel "akernel" and with given width.
        n=size(x,"*")
        npoints=size(xi,"*")
        //
        // 1. Reshape x (the data) into a matrix, with 
        // one row for each point, and one column for each data
        x=repmat(x',npoints,1);
        //
        // 2. Reshape xi (the points where to evaluate f)
        // into a matrix, with 
        // one row for each point, and one column for each data
        xpoints=repmat(xi,1,n);
        //
        u = (xpoints-x)/width; // Standardized parameter
        if (func=="pdf") then
            K=ksdensity_kernelpdf(u,akernel); // Evaluate PDF Kernel
            f=sum(K,"c")/(n*width); // Sum
        else
            K=ksdensity_kernelcdf(u,akernel); // Evaluate CDF Kernel
            f=sum(K,"c")/n; // Sum
        end
        //
        if (support=="positive") then
            f(xi<0) = 0
        end
    endfunction

    rhs = nargin
    if rhs == 0 then
        error(msprintf(_("%s: Wrong number of input argument: At least %d expected.\n"), "kdensity", 1));
    end


    if rhs == 1 then
        x = varargin(1);
        [f, xi, width] = %kdensity(x);
    elseif rhs == 2 then
        x = varargin(1);
        xi = varargin(2);
        [f, xi, width] = %kdensity(x, xi);
    else
        x = varargin(1);
        startOptions = 3;
        if modulo(rhs, 2) == 1 then
            xi = [];
            startOptions = 2;
        else
            xi = varargin(2);
        end

        akernel = "normal";
        npoints = 100;
        support = "unbounded";
        width = []; //n = size(x, "*"); 1.0600000000000001*stdev(x)*(n^(-1/5));
        func = "pdf";

        for i = startOptions:2:rhs
            option = varargin(i);
            select option
            case "kernel"
                akernel = varargin(i+1);
            case "npoints"
                npoints = varargin(i+1);
            case "support"
                support = varargin(i+1);
            case "width"
                width = varargin(i+1);
            case "func"
                func = varargin(i+1);
            else
                error(msprintf(_("%s: Wrong value for input argument #%d: %s expected.\n"), "ksdensity", i, sci2exp(["kernel", "npoints", "support", "width", "func"])));
            end
        end

        [f, xi, width] = %kdensity(x, xi, akernel, npoints, support, width, func);
    end
   
endfunction