// Copyright (C) 2013 - 2016 - Michael Baudin
//
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt
function plotmatrix(varargin)
    // Plot an X vx Y scatter plot matrix
    //
    // Calling Sequence
    //   plotmatrix(x)
    //   plotmatrix(x,y)
    //   plotmatrix(x,y,key1,value1,key2,value2,...)
    //   plotmatrix(x,y,"xlabels",xlabels)
    //   plotmatrix(x,y,"ylabels",ylabels)
    //   plotmatrix(x,y,"ptsize",ptsize)
    //   plotmatrix(x,y,"valuelabels",valuelabels)
    //   plotmatrix(x,y,"symbol",symbol)
    //   plotmatrix(x,"histogram",histogram)
    //   plotmatrix(x,"nbclasses",nbclasses)
    //   plotmatrix(x,"histoYlabel",histoYlabel)
    //
    // Parameters
    //   x : a n-by-ninput matrix of doubles, the x datas
    //   y : a n-by-noutput matrix of doubles, the y datas
    //   xlabels : a ninput-by-1 matrix of strings, the x labels (default="")
    //   ylabels : a noutput-by-1 matrix of strings, the y labels (default="")
    //   ptsize : a 1-by-1 matrix of doubles, integer value, positive, the number pixels for the dots (default ptsize=2)
    //   valuelabels : a 1-by-1 matrix of booleans, set to true to print the x and y value labels (default=%t)
    //   symbol : a 1-by-1 matrix of strings, the point symbols (default="b.")
    //   histogram : a 1-by-1 matrix of booleans, set to true to print the histogram (default=%t)
    //   nbclasses : a 1-by-1 matrix of doubles, integer value, positive, the number of classes in the histogram (default nbclasses=[])
    //   histoYlabel : a 1-by-1 matrix of strings, the Y-label in the histogram (default histoYlabel="Frequency")
    //
    // Description
    // Plots a matrix of scatter plots representing the 
    // dependencies of Y vs X: plots the columns of Y versus the columns of X. 
    // This creates a rectangular matrix of plots with ninput columns and 
    // noutput rows.
    //
    // If only X is specified, plots the columns of X versus the columns of X, 
    // and replaces the diagonal with histograms. 
    // This creates a square matrix of plots with ninput columns and ninput rows.
    // The plotmatrix function is partly compatible with Matlab.
    //
    // Examples
    // // Example 1
    // // Plot Y versus X
    // m=1000;
    // x1=distfun_unifrnd(0,1,m,1);
    // x2=distfun_unifrnd(0,1,m,1);
    // x3=distfun_unifrnd(0,1,m,1);
    // y1=2*x1.*x2+x3;
    // y2=-3*x1+x2.^2-2*x3;
    // y3=sin(x1)-3*x2+3*x3;
    // x=[x1,x2,x3];
    // y=[y1,y2,y3];
    // //
    // xlabels=["X1","X2","X3"];
    // ylabels=["Y1","Y2","Y3"];
    // // No labels
    // scf();
    // plotmatrix(x,y);
    // // With labels (Figure 1)
    // scf();
    // plotmatrix(x,y,"xlabels",xlabels,"ylabels",ylabels);
    // // Without XY value labels
    // scf();
    // plotmatrix(x,y,"valuelabels",%f);
    // // Without XY value labels, and XY labels
    // scf();
    // plotmatrix(x,y,"valuelabels",%f,..
    // "xlabels",xlabels,"ylabels",ylabels);
    // // Set the point size
    // scf();
    // plotmatrix(x,y,"ptsize",1);
    // // With red crosses
    // scf();
    // plotmatrix(x,y,"symbol","rx");
    // // 
    // // Example 2
    // // Plot Y versus X
    // m=1000;
    // x1=distfun_normrnd(0,1,m,1);
    // x2=distfun_unifrnd(-1,1,m,1);
    // y1=x1.^2+x2;
    // y2=-3*x1+x2.^2;
    // y3=x1-3*exp(x2);
    // x=[x1,x2];
    // y=[y1,y2,y3];
    // //
    // xlabels=["X1","X2"];
    // ylabels=["Y1","Y2","Y3"];
    // // No labels
    // scf();
    // plotmatrix(x,y);
    // // With labels, and red circles
    // scf();
    // plotmatrix(x,y,"xlabels",xlabels,"ylabels",ylabels,..
    // "symbol","ro");
    // // 
    // // Example 3
    // // Plot X versus X
    // m=1000;
    // x1=distfun_unifrnd(0,1,m,1);
    // x2=distfun_unifrnd(0,1,m,1);
    // x3=distfun_unifrnd(0,1,m,1);
    // y1=2*x1.*x2+x3;
    // y2=-3*x1+x2.^2-2*x3;
    // y3=sin(x1)-3*x2+3*x3;
    // y=[y1,y2,y3];
    // //
    // ylabels=["Y1","Y2","Y3"];
    // // No labels
    // scf();
    // plotmatrix(y);
    // // With labels (Figure 2)
    // scf();
    // plotmatrix(y,"xlabels",ylabels);
    // // With labels, without value labels
    // scf();
    // plotmatrix(y,"xlabels",ylabels,"valuelabels",%f);
    // // With labels, without value labels, with red circles
    // scf();
    // plotmatrix(y,"xlabels",ylabels,"valuelabels",%f,..
    // "symbol","ro");
    // // With labels, without value labels, with red dots, 
    // // with symbols of size 1
    // scf();
    // plotmatrix(y,"xlabels",ylabels,"valuelabels",%f,..
    // "symbol","r.","ptsize",1);
    // // With the histogram
    // scf();
    // plotmatrix(y,"histogram",%t);
    // // With the histogram, and the labels
    // scf();
    // plotmatrix(y,"histogram",%t,"xlabels",ylabels);
    // 
    // Authors
    // Copyright (C) 2013 - 2016 - Michael Baudin

    function %plotmatrix(x, y, xlabels, ylabels, ptsize, valuelabels, symbol, histogram, nbclasses, histoYlabel)
        arguments
            x {mustBeA(x, "double")}
            y {mustBeA(y, "double")}
            xlabels {mustBeA(xlabels, ["string", "empty"])} = []
            ylabels {mustBeA(ylabels, ["string", "empty"])} = []
            ptsize (1,1) {mustBeA(ptsize, "double"), mustBeInteger, mustBeGreaterThanOrEqual(ptsize, 1)} = 2
            valuelabels (1,1) {mustBeA(valuelabels, "boolean")} = %t
            symbol  (1,1) {mustBeA(symbol, "string")} = "b."
            histogram (1,1) {mustBeA(histogram, "boolean")} = %t
            nbclasses {mustBeA(nbclasses, "double"), mustBeScalarOrEmpty, mustBeInteger, mustBeGreaterThanOrEqual(nbclasses, 1)} = []
            histoYlabel (1,1) {mustBeA(histoYlabel, "string")} = "Frequency"
        end

        [n, ninput] =size(x);
        noutput = size(y, "c");

        if size(y, "r") <> n then
            error(msprintf(_("%s: Wrong size for input argument #%d: Must have the same number of rows as #%d.\n"), "plotmatrix", 2, 1));
        end

        if (xlabels<>[]) & size(xlabels, "*") <> ninput then
            error(msprintf(_("%s: Wrong size for input argument ""%s"": A vector of length %d expected.\n"), "plotmatrix", "xlabels", ninput));
        end
        if (ylabels<>[]) & size(ylabels, "*") <> noutput then
            error(msprintf(_("%s: Wrong size for input argument ""%s"": A vector of length %d expected.\n"), "plotmatrix", "ylabels", noutput));
        end
            
        if (plotmode=="XY") then
            iserr=%f
            if (xlabels<>[]&ylabels==[]) then
                iserr=%t
            end
            if (xlabels==[]&ylabels<>[]) then
                iserr=%t
            end
            if (iserr) then
                errmsg=gettext("%s: xlabels and ylabels are both expected.")
                error(msprintf(errmsg,"plotmatrix"))
            end
        end
        //
        drawlater()
        for j=1:noutput
            for i=1:ninput
                p=(j-1)*ninput + i;
                subplot(noutput,ninput,p);
                if (i==j&plotmode=="XX") then
                    if (histogram) then
                        if (nbclasses==[]) then
                            histo(x(:,i))
                        else
                            histo(x(:,i),nbclasses)
                        end
                        if (xlabels<>[]) then
                            xlabel(xlabels(i))
                        end
                        ylabel(histoYlabel)
                    else
                        if (xlabels<>[]) then
                            xstring(0.5,0.5,xlabels(i))
                        end
                    end
                else
                    plot(x(:,i),y(:,j),symbol);
                    e=gce();
                    e.children.mark_size=ptsize;
                    if (xlabels<>[]&ylabels<>[]) then
                        xlabel(xlabels(i))
                        ylabel(ylabels(j))
                    end
                    if (~valuelabels) then
                        a=gca()
                        a.auto_ticks(1:3)="off";
                    end
                end
            end
        end
        drawnow()
    endfunction
   
    rhs = nargin;
    if rhs == 0 then
        error(msprintf(_("%s: Wrong number of input argument: At least %d expected.\n"), "plotmatrix", 1));
    end

    if rhs == 1 then
        x = varargin(1);
        plotmode="XX"
        %plotmatrix(x, x)
    elseif rhs == 2 then
        x = varargin(1);
        y = varargin(2);
        plotmode="XY"
        %plotmatrix(x, y)
    else
        x = varargin(1);
        startOptions = 3;
        if modulo(rhs, 2) == 1 then
            y = x;
            startOptions = 2;
            plotmode="XX"
        else
            y = varargin(2);
            plotmode="XY"
        end

        xlabels = []
        ylabels = []
        ptsize = 2
        valuelabels = %t
        symbol = "b."
        histogram = %t
        nbclasses = []
        histoYlabel = "Frequency"

        for i = startOptions:2:rhs
            option = varargin(i);
            select option
            case "xlabels"
                xlabels = varargin(i+1);
            case "ylabels"
                ylabels = varargin(i+1);
            case "ptsize"
                ptsize = varargin(i+1);
            case "valuelabels"
                valuelabels = varargin(i+1);
            case "symbol"
                symbol = varargin(i+1);
            case "histogram"
                histogram = varargin(i+1);
            case "nbclasses"
                nbclasses = varargin(i+1);
            case "histoYlabel"
                histoYlabel = varargin(i+1);
            else
                error(msprintf(_("%s: This ""%s"" option is not managed.\n"), "plotmatrix", option));
            end
        end

        %plotmatrix(x, y, xlabels, ylabels, ptsize, valuelabels, symbol, histogram, nbclasses, histoYlabel);
    end    
endfunction
